@@ -1,32 +1,17 @@
-import os
 import numpy
-import json
-import pycifstar
 from cryspy.A_functions_base.orbital_functions import calc_jl
+from cryspy.A_functions_base.database import DATABASE
 
-f_dir = os.path.dirname(__file__)
 
-with open(os.path.join(f_dir, "atom_electron_configuration.txt")) as fid:
-    data = fid.read()
+def get_n_zeta_coeff_for_atom_with_orbital(atom_name: str, orbital_name: str):
+    d_radial_slater = DATABASE["Orbitals by radial Slater functions"]
+    n, zeta, coeff = d_radial_slater[(atom_name.capitalize(), orbital_name.lower())]
+    return n, zeta, coeff
 
-D_ATOM_ELECTRON_CONFIGURATION = json.loads(data)
-
-f_atom_rho_orbital_radial_slater = os.path.join(f_dir, "atom_rho_orbital_radial_slater.rcif")
-DATA_ATOM_RHO_ORBITAL_RADIAL_SLATER = pycifstar.to_data(f_atom_rho_orbital_radial_slater)
-
-f_scattering_amplitude = os.path.join(f_dir, "table_6.1.1.4.txt")
-D_SCATTERING_AMPLITUDE = {}
-for line in numpy.loadtxt(f_scattering_amplitude, skiprows=1, dtype=str):
-    D_SCATTERING_AMPLITUDE[str(line[0])] = line[2:11].astype(float)
-
-f_dispersion = os.path.join(f_dir, "table_4.2.6.8.txt")
-D_DISPERSION = {}
-with open(f_dispersion, "r") as fid:
-    D_DISPERSION["table_wavelength"] = (numpy.array(fid.readline().strip().split()[2:], dtype=float))[::-1]
-for line in numpy.loadtxt(f_dispersion, skiprows=1, dtype=str):
-    D_DISPERSION[str(line[0])] = (line[2:12].astype(float) + 1j*line[13:23].astype(float))[::-1]
-
-def get_atom_name_ion_charge_shell(ion_name: str):
+def get_atomic_symbol_ion_charge_isotope_number_by_ion_symbol(ion_name: str):
+    """
+    Example: ion_name = "13C2-"
+    """
     ion_name_sh = ion_name.strip().lower()
     flag_isotope, flag_charge = False, False
     isotope_number = ""
@@ -56,7 +41,10 @@ def get_atom_name_ion_charge_shell(ion_name: str):
     else:
         atom_name = ion_name_sh[isotope_position:].capitalize()
         ion_charge = 0
+    return atom_name, ion_charge, isotope_number
 
+def get_atom_name_ion_charge_shell(ion_name: str):
+    atom_name, ion_charge, isotop_number = get_atomic_symbol_ion_charge_isotope_number_by_ion_symbol(ion_name)
     full_shell = get_full_shell(atom_name, ion_charge)
     return atom_name, ion_charge, full_shell
 
@@ -110,7 +98,8 @@ def get_shell_orbit_electrons(orbital:str):
 
 
 def get_full_shell(atom_name: str, ion_charge: int = 0):
-    core_sh = D_ATOM_ELECTRON_CONFIGURATION[atom_name]
+    d_elements = DATABASE["Elements"]
+    core_sh = d_elements["Orbital occupancy", atom_name]
     l_shell = core_sh.split("]")
     if len(l_shell) == 1:
         shell_full = l_shell[0].strip()
@@ -123,27 +112,27 @@ def get_full_shell(atom_name: str, ion_charge: int = 0):
         shell_full = shell_start_full + " " + shell_end
     return shell_full
 
-L_PREDEFINED_NAME = [loop.name for loop in DATA_ATOM_RHO_ORBITAL_RADIAL_SLATER.loops]
-
 
 def calc_jl_for_shell_of_ion(sthovl, ion_name: str, shell: str, kappa:float = 1., l_max:int=5):
     n_shell, orbit_name = shell[0], shell[1]
     atom_name, ion_charge, full_shell = get_atom_name_ion_charge_shell(ion_name)
     n_total_el = sum([get_shell_orbit_electrons(orbital)[2] for orbital in full_shell.split()])
     jl_res = None
+    d_radial_slater = DATABASE["Orbitals by radial Slater functions"]
     for orbital in shell.strip().split():
         n_shell, orbit_name, n_el = get_shell_orbit_electrons(orbital)
         s_name = f"{atom_name.capitalize()}_{orbit_name.lower()}"
         shell = f"{n_shell:}{orbit_name:}"
         try:
-            ind_name = L_PREDEFINED_NAME.index(s_name)
-            loop = DATA_ATOM_RHO_ORBITAL_RADIAL_SLATER.loops[ind_name]
-            n = numpy.array(loop["atom_rho_orbital_radial_slater_n0"], dtype=int)
-            zeta = numpy.array(loop["atom_rho_orbital_radial_slater_zeta0"], dtype=float)
-            coeff = numpy.array(loop[f"atom_rho_orbital_radial_slater_coeff_{shell:}"], dtype=float)
-            if numpy.any(numpy.isnan(coeff)):
-                raise ValueError
-        except ValueError:
+            n, zeta, coeff = d_radial_slater[(atom_name.capitalize(), shell.lower())]
+            # 
+            # d_atom = D_ATOM_RHO_ORBITAL_RADIAL_SLATER[atom_name.capitalize(), orbit_name.lower()]
+            # n = d_atom["n0"]
+            # zeta = d_atom["zeta0"]
+            # coeff = d_atom[shell]
+            # if numpy.any(numpy.isnan(coeff)):
+            #     raise KeyError
+        except KeyError: #FIXME
             n = numpy.array([n_shell,], dtype=int)
             zeta = numpy.array([n_total_el,], dtype=float)
             coeff = numpy.array([1,], dtype=float)
@@ -164,7 +153,8 @@ def calc_jl_for_ion(sthovl, ion_name: str, kappa:float = 1., l_max:int=5):
 
 
 def calc_scattering_amplitude_tabulated(ion_name: str, sthovl, flag_sthovl: bool = False):
-    params = D_SCATTERING_AMPLITUDE[ion_name]
+    d_scattering_amplitude = DATABASE["Scattering amplitude"]
+    params = d_scattering_amplitude[ion_name.capitalize()]
     a1, b1 = params[0], params[1]
     a2, b2 = params[2], params[3]
     a3, b3 = params[4], params[5]
